/**
 * 
 */
package org.richfaces.testng;

import org.richfaces.RichSeleniumTest;
import org.richfaces.SeleniumTestBase;
import org.richfaces.Templates;
import org.testng.Assert;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;

/**
 * @author Andrey Markavstov
 * 
 */
public class AjaxPollTest extends SeleniumTestBase implements RichSeleniumTest {

	public AjaxPollTest() {
		super("http", "localhost", serverPort);
	}

	/**
	 * This method are invoking before selenium tests started
	 */
	@BeforeTest
	@Parameters( { "browser" })
	public void startSelenium(String browser) {
		super.startSelenium(browser);
	}

	/**
	 * This method are invoking after selenium tests completed
	 */
	@AfterTest
	public void stopSelenium() {
		super.stopSelenium();
	}
	
	@Test
	public void testAjaxPollComponent() {
		_testAjaxPollComponent(Templates.SIMPLE);
		_testAjaxPollComponent(Templates.DATATABLE);
		_testAjaxPollComponent(Templates.MODALPANEL);
	}
	
	private void _testAjaxPollComponent(Templates template) {
		renderPage(template);
		String parentId = getParentId() + "_form:";
		String pollId = parentId + "poll";
		String inputId = parentId + "_value";
		
		enablePoll(parentId + "_enabled");
		AssertValueEquals(inputId, "1");
		
		writeStatus("Polling in progress...");
		pause(1500, pollId);
		AssertValueNotEquals(inputId, "1", "Polling does not fire after component was enabled. Or 'ReRender' attribute does not work");

		waiteForCondition("document.getElementById('"+inputId+"').value == 8", 7000);
		
		pause(1500, pollId);
		writeStatus("Polling should be stopped...");
		
		AssertValueEquals(inputId, "8", "Polling should be stopped. It continues to poll after component was disabled.");
		AssertTextEquals(parentId + "_text", "Polling", "Polling terminated abnormal or invalid count of ajax requests has been submitted");

	}
	
	private void enablePoll(String id) {
		writeStatus("Enable polling...");
		try {
			clickById(id);
			waitForAjaxCompletion();
		}catch (Exception e) {
			Assert.fail("Poll component failed after attempt to enable polling. Caused by " + e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.ajax4jsf.test.base.RichSeleniumTest#getTestUrl()
	 */
	public String getTestUrl() {
		return "/faces/pages/ajaxPoll/ajaxPollTest.xhtml";
	}

}
