package org.richfaces.testng;

import org.richfaces.RichSeleniumTest;
import org.richfaces.SeleniumTestBase;
import org.richfaces.Templates;
import org.testng.Assert;
import org.testng.annotations.AfterTest;
import org.testng.annotations.BeforeTest;
import org.testng.annotations.Parameters;
import org.testng.annotations.Test;


public class InputNumberSpinnerTest extends SeleniumTestBase implements RichSeleniumTest {

    public InputNumberSpinnerTest() {
        super("http", "localhost", serverPort);
    }
    
    /**
     * This method are invoking before selenium tests started
     */
    @BeforeTest
    @Parameters( { "browser" })
    public void startSelenium(String browser) {
        super.startSelenium(browser);
    }

    /**
     * This method are invoking after selenium tests completed
     */
    @AfterTest
    public void stopSelenium() {
        super.stopSelenium();
    }

    @Test
    public void testInputNumberSpinnerComponent() throws Exception {
        _testInputNumberSpinnerComponent(Templates.SIMPLE);
        //it looks as if the spinner does not work inside rich:dataTable
        _testInputNumberSpinnerComponent(Templates.DATATABLE);
        _testInputNumberSpinnerComponent(Templates.MODALPANEL);
    }

    private void _testInputNumberSpinnerComponent(Templates template) {
        renderPage(getTestUrl(), template);

        writeStatus("check whether the spinner is invariant with respect to pair of sequential opposite operations");
        
        //start value = 20
        String before = getSpinnerValue();
        clickUp();
        clickDown();
        String after = getSpinnerValue();
        Assert.assertEquals(before, after);
        Assert.assertEquals(after, "20");

        writeStatus("check upper bound and cycling");

        clickUp(); //+10
        clickUp(); //+10
        clickUp(); //+10
        clickUp(); //+10
        clickUp(); //+10
        Assert.assertEquals(getSpinnerValue(), "10");

        writeStatus("check lower bound and cycling");

        clickDown(); //-10
        Assert.assertEquals(getSpinnerValue(), "0");
        clickDown(); //-10
        Assert.assertEquals(getSpinnerValue(), "50");
        clickDown(); //-10
        Assert.assertEquals(getSpinnerValue(), "40");
    }

    private void clickUp() {
        String id = getParentId() + "_form:" + "spinnerButtons";
        selenium.fireEvent("xpath=//table[@id='" + id + "']/tbody/tr[1]/td/input", "mousedown");
        selenium.fireEvent("xpath=//table[@id='" + id + "']/tbody/tr[1]/td/input", "mouseup");
    }

    private void clickDown() {
        String id = getParentId() + "_form:" + "spinnerButtons";
        selenium.fireEvent("xpath=//table[@id='" + id + "']/tbody/tr[2]/td/input", "mousedown");
        selenium.fireEvent("xpath=//table[@id='" + id + "']/tbody/tr[2]/td/input", "mouseup");
    }

    private String getSpinnerValue() {
        String id = getParentId() + "_form:" + "spinnerEdit";
        return runScript("$('" + id + "').down('input').value");
    }

	public String getTestUrl() {
		return "/faces/pages/inputNumberSpinner/inputNumberSpinnerTest.xhtml";
	}

}
